package legion.muyue.best2048;

import android.os.Bundle;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.animation.AnimationUtils; // Ajout pour Animation
import android.widget.TextView;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;
import androidx.gridlayout.widget.GridLayout;
import android.widget.Button; // Ajout pour Button
import androidx.core.view.WindowCompat;
import androidx.core.view.WindowInsetsCompat;
import androidx.core.view.WindowInsetsControllerCompat;


public class MainActivity extends AppCompatActivity {

    private GridLayout gameBoardLayout;
    private TextView scoreTextView;
    private TextView highScoreTextView;
    private Game game;

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        EdgeToEdge.enable(this);
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        gameBoardLayout = findViewById(R.id.gameBoard);
        scoreTextView = findViewById(R.id.scoreLabel);
        highScoreTextView = findViewById(R.id.highScoreLabel);
        Button restartButton = findViewById(R.id.restartButton); // Initialisation
        Button statsButton = findViewById(R.id.statsButton); // Initialisation
        Button menuButton = findViewById(R.id.menuButton); // Initialisation
        Button multiplayerButton = findViewById(R.id.multiplayerButton); // Initialisation

        game = new Game();

        initGameBoardLayout();

        // Initialisation du jeu et de l'UI via restartGame
        restartGame();

        // Ajout des listeners de swipe
        setupSwipeListener();

        //Listeners des boutons
        multiplayerButton.setOnClickListener(v -> {
            v.startAnimation(AnimationUtils.loadAnimation(this, R.anim.button_press));
            showMultiplayerScreen();
        });
        restartButton.setOnClickListener(v -> restartGame()); // Lambda pour restart
        statsButton.setOnClickListener(v -> showStats()); // Lambda pour stats
        menuButton.setOnClickListener(v -> showMenu()); // Lambda pour menu
    }

    private void initGameBoardLayout() {
        gameBoardLayout.removeAllViews();
        gameBoardLayout.setColumnCount(4);
        gameBoardLayout.setRowCount(4);
    }

    private void updateUI() {
        gameBoardLayout.removeAllViews(); // Efface les tuiles précédentes

        for (int x = 0; x < 4; x++) {
            for (int y = 0; y < 4; y++) {
                TextView tileTextView = new TextView(this);
                int value = game.getGameBoard(x, y);

                // Utilisation d'une méthode pour définir l'arrière-plan et le texte
                setTileAppearance(tileTextView, value);

                GridLayout.LayoutParams params = new GridLayout.LayoutParams();
                params.width = 0;
                params.height = 0;
                params.rowSpec = GridLayout.spec(x, 1f);
                params.columnSpec = GridLayout.spec(y, 1f);
                // Utilisation des dimensions pour les marges
                params.setMargins(
                        (int) getResources().getDimension(R.dimen.tile_margin),
                        (int) getResources().getDimension(R.dimen.tile_margin),
                        (int) getResources().getDimension(R.dimen.tile_margin),
                        (int) getResources().getDimension(R.dimen.tile_margin)
                );
                tileTextView.setLayoutParams(params);

                gameBoardLayout.addView(tileTextView);
            }
        }
        // Met à jour les TextViews du score et du high score en utilisant getString
        scoreTextView.setText(getString(R.string.score, game.getScore()));
        highScoreTextView.setText(getString(R.string.high_score, game.getHighScore()));

    }

    // Méthode refactorisée pour l'apparence des tuiles
    private void setTileAppearance(TextView tile, int value) {
        // int drawableId; // N'est plus utilisé directement
        int textColorId;
        int textSizeId;

        // Utilisation d'un switch pour déterminer la couleur de fond, texte et taille.
        switch (value) {
            case 2:
                tile.setBackgroundResource(R.drawable.tile_background); // Utilise le drawable générique
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_2_color)); // Applique la teinte
                textColorId = R.color.text_tile_low;
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 4:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_4_color));
                textColorId = R.color.text_tile_low;
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 8:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_8_color));
                textColorId = R.color.text_tile_low; // Couleur texte change à partir de 8 dans le code final, mais pas dans ce snippet
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 16:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_16_color));
                textColorId = R.color.text_tile_low; // Devrait être text_tile_high selon la logique standard 2048
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 32:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_32_color));
                textColorId = R.color.text_tile_low; // Devrait être text_tile_high
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 64:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_64_color));
                textColorId = R.color.text_tile_low; // Devrait être text_tile_high
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 128:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_128_color));
                textColorId = R.color.text_tile_high; // Correct ici
                textSizeId = R.dimen.text_size_tile_medium; // Taille change
                break;
            case 256:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_256_color));
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_medium;
                break;
            case 512:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_512_color));
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_medium;
                break;
            case 1024:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_1024_color));
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_large; // Taille change
                break;
            case 2048:
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_2048_color));
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_large;
                break;
            default: // Tuile vide ou > 2048
                tile.setBackgroundResource(R.drawable.tile_background);
                tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_empty_color));
                textColorId = android.R.color.transparent; // Pas de texte
                textSizeId = R.dimen.text_size_tile_small; // Taille par défaut
                // Gérer les tuiles > 2048 si nécessaire (non montré dans ce snippet)
                if (value > 2048) {
                    tile.getBackground().setTint(ContextCompat.getColor(this, R.color.tile_super_color));
                    textColorId = R.color.text_tile_high;
                    textSizeId = R.dimen.text_size_tile_large;
                }
                break;
        }

        if (value > 0) {
            tile.setText(String.valueOf(value));
            tile.setTextColor(ContextCompat.getColor(this, textColorId));
            // Utilise la dimension pour la taille du texte
            tile.setTextSize(TypedValue.COMPLEX_UNIT_PX, getResources().getDimension(textSizeId));
        } else {
            tile.setText(""); // Efface le texte pour les tuiles vides
        }
        tile.setGravity(Gravity.CENTER);
    }


    private void setupSwipeListener() {
        // Utilise l'interface SwipeListener
        gameBoardLayout.setOnTouchListener(new OnSwipeTouchListener(MainActivity.this, new OnSwipeTouchListener.SwipeListener() {
            @Override
            public void onSwipeTop() {
                handleSwipe(Direction.UP);
            }

            @Override
            public void onSwipeBottom() {
                handleSwipe(Direction.DOWN);
            }

            @Override
            public void onSwipeLeft() {
                handleSwipe(Direction.LEFT);
            }

            @Override
            public void onSwipeRight() {
                handleSwipe(Direction.RIGHT);
            }
        }));
    }

    // Méthode pour gérer les swipes via l'enum
    private void handleSwipe(Direction direction) {
        switch (direction) {
            case UP:
                game.pushUp();
                break;
            case DOWN:
                game.pushDown();
                break;
            case LEFT:
                game.pushLeft();
                break;
            case RIGHT:
                game.pushRight();
                break;
        }
        game.addNewNumbers();
        updateUI();
    }

    // Méthode pour redémarrer le jeu
    private void restartGame() {
        game = new Game(); // Crée une nouvelle instance de Game
        game.addNewNumbers(); // Ajoute les tuiles initiales
        game.addNewNumbers();
        updateUI(); // Met à jour l'interface utilisateur
    }

    // Stubs pour les autres boutons
    private void showStats() {
        //A faire
    }
    private void showMenu() {
        //A faire
    }

    private void showMultiplayerScreen() {
        //A faire
    }

    // Enum pour les directions
    private enum Direction {
        UP, DOWN, LEFT, RIGHT
    }

}